;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Here's the Fibonacci function, defined recursively:
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fib(n)
  "
;; Examples:
 (fib 5) ;; 5
 (mapcar #'fib (iseq 1 8)) ;; (1 1 2 3 5 8 13 21)
 (terpri)
 (time (fib 20))
 (terpri)
 (time (fib 30))
 (trace fib)                                                                                                                                        (fib 8)                                                                                                                                            (untrace fib)                                                                                                                                     
"
  (if 
      (not 
       (and 
	(integerp n) 
	(> n 0))) 
      (error "Only natural numbers are allowed")
    (case n
	  ;; the following two cases are the basis cases:
	  (1 1)
	  (2 1)
	  ;; and, if we're not in a basis case, then we should use recursion. 
	  ;; This means that function fib actually invokes itself:
	  (t (+ (fib (- n 1)) (fib (- n 2))))
	  ;; but, because the argument decreases, we'll eventually hit the
	  ;; ``basement''. 
	  )
    )
  )

;;
;; http://www.cs.cmu.edu/~rwh/introsml/core/recfns.htm
;; suggests a better recursive function:
;; for n>=0, fibb n evaluates to (a, b), where a is the nth Fibonacci number
;;   and b is the (n-1)st *)
(defun fibb(n)
  "
;; Examples:
 (fibb 5) ;; (5 3) 
 (mapcar #'fibb (iseq 1 8)) ;; ((1 0) (1 1) (2 1) (3 2) (5 3) (8 5) (13 8) (21 13))
 (terpri)
 (time (fibb 20))
 (terpri)
 (time (fibb 30))
 (terpri)
 (time (fibb 100))
 (time (fibb 300))
"
  (if 
      (not 
       (and 
	(integerp n) 
	(> n 0))) 
      (error "Only natural numbers are allowed")
    (case n
	  (1 '(1 0))
	  (2 '(1 1))
	  (t (let ((temp (fibb (- n 1))))
	       (list (sum temp) (first temp))
	       )
	     )
	  )
    )
  )
(try-out fib)
(try-out fibb)
